const { Tracer } = require('@aws-lambda-powertools/tracer');
const AWSXray = require('aws-xray-sdk-core');
const AWS = require('aws-sdk');

// If running in Docker: docker run -p 8000:8000 -d amazon/dynamodb-local -jar DynamoDBLocal.jar -sharedDb -dbPath .
const TABLE_NAME = 'TestTableMulti'; // Create aws dynamodb create-table --cli-input-json file://createTable.json --endpoint-url http://localhost:8000
const CLIENT_CONFIG = {
  region: 'us-east-1',
  endpoint: 'http://localhost:8000', // Remove this if testing against actual DynamoDB table
}

const tracer = new Tracer();

const scan = async (client) => {
  const params = {
    TableName: TABLE_NAME,
  };
  await client.scan(params).promise();
};

const put = async (client, id) => {
  const params = {
    TableName: TABLE_NAME,
    Item: {
      sk: `param-${id}`,
      value: '1',
      id: 'someValue'
    },
  };
  await client.put(params).promise();
};

const deleteOne = async (client, id) => {
  const params = {
    TableName: TABLE_NAME,
    Key: {
      id: 'someValue',
      sk: `param-${id}`,
    },
  };
  await client.delete(params).promise();
};

const get = async (client, id) => {
  const params = {
    TableName: TABLE_NAME,
    Key: {
      id: 'someValue',
      sk: `param-${id}`,
    },
  };
  await client.get(params).promise();
};

const clientOnly = async () => {
  console.log('const client = new AWS.DynamoDB.DocumentClient({ ...CLIENT_CONFIG });');
  const client = new AWS.DynamoDB.DocumentClient({ ...CLIENT_CONFIG });
  console.log('client instanceof(AWS.DynamoDB.DocumentClient) should be true, actual:', client instanceof(AWS.DynamoDB.DocumentClient));
  console.log('client instanceof(AWS.DynamoDB) should be false, actual:', client instanceof(AWS.DynamoDB));
  await scan(client);
  console.log('scan OK');
  await put(client, 1);
  console.log('put OK');
  await get(client, 1);
  console.log('get OK');
  await deleteOne(client, 1);
  console.log('delete OK');
};

const sdkTracedBroken = async () => {
  console.log('const client = AWSXray.captureAWSClient(new AWS.DynamoDB.DocumentClient({ ...CLIENT_CONFIG }).service);');
  const client = AWSXray.captureAWSClient(new AWS.DynamoDB.DocumentClient({ ...CLIENT_CONFIG }).service);
  console.log('client instanceof(AWS.DynamoDB.DocumentClient) should be true, actual:', client instanceof(AWS.DynamoDB.DocumentClient));
  console.log('client instanceof(AWS.DynamoDB) should be false, actual:', client instanceof(AWS.DynamoDB));
  await scan(client);
  console.log('scan OK');
  await put(client, 2);
  console.log('put OK');
  await get(client, 2);
  console.log('get OK');
  await deleteOne(client, 2);
  console.log('delete OK');
};

const sdkTracedOk = async () => {
  console.log('const client = new AWS.DynamoDB.DocumentClient({ ...CLIENT_CONFIG }); \nAWSXray.captureAWSClient(client.service);');
  const client = new AWS.DynamoDB.DocumentClient({ ...CLIENT_CONFIG });
  AWSXray.captureAWSClient(client.service);
  console.log('client instanceof(AWS.DynamoDB.DocumentClient) should be true, actual:', client instanceof(AWS.DynamoDB.DocumentClient));
  console.log('client instanceof(AWS.DynamoDB) should be false, actual:', client instanceof(AWS.DynamoDB));
  await scan(client);
  console.log('scan OK');
  await put(client, 2);
  console.log('put OK');
  await get(client, 2);
  console.log('get OK');
  await deleteOne(client, 2);
  console.log('delete OK');
};

const tracerTracedHotFix = async () => {
  const client = tracer.captureAWSClient(new AWS.DynamoDB.DocumentClient({ ...CLIENT_CONFIG }));
  console.log(client instanceof(AWS.DynamoDB.DocumentClient));
  console.log('client instanceof(AWS.DynamoDB.DocumentClient) should be true, actual:', client instanceof(AWS.DynamoDB.DocumentClient));
  console.log('client instanceof(AWS.DynamoDB) should be false, actual:', client instanceof(AWS.DynamoDB));
  await scan(client);
  console.log('scan OK');
  await put(client, 5);
  console.log('put OK');
  await get(client, 5);
  console.log('get OK');
  await deleteOne(client, 5);
  console.log('delete OK');
};


module.exports = handler = async () => {
  await clientOnly();
  console.log('\n\n');
  await sdkTracedOk();
  console.log('\n\n');
  await tracerTracedHotFix(); 
};

handler();